# encoding: utf-8
# WARNING! Do not edit! https://waf.io/book/index.html#_obtaining_the_waf_file

import os
import pipes
import subprocess
import sys
from waflib import Logs,Task,Context
from waflib.Tools.c_preproc import scan as scan_impl
from waflib.TaskGen import feature,before_method
def join_args(args):
	return" ".join(pipes.quote(arg)for arg in args)
def configure(cfg):
	cfg.load("compiler_cxx")
	cfg.load("python")
	cfg.check_python_version(minver=(2,7))
	if not cfg.env.LLVM_CONFIG:
		cfg.find_program("llvm-config",var="LLVM_CONFIG")
	if not cfg.env.GENPYBIND:
		cfg.find_program("genpybind",var="GENPYBIND")
	cfg.env.GENPYBIND_RESOURCE_DIR=os.path.join(cfg.cmd_and_log(cfg.env.LLVM_CONFIG+["--libdir"]).strip(),"clang",cfg.cmd_and_log(cfg.env.LLVM_CONFIG+["--version"]).strip())
	if os.path.exists(cfg.env.GENPYBIND_RESOURCE_DIR):
		cfg.msg("Checking clang resource dir",cfg.env.GENPYBIND_RESOURCE_DIR)
	else:
		cfg.fatal("Clang resource dir not found")
@feature("genpybind")
@before_method("process_source")
def generate_genpybind_source(self):
	module=getattr(self,"module",self.target)
	out="genpybind-%s.%d.cpp"%(module,self.idx)
	out=self.path.get_bld().find_or_declare(out)
	task=self.create_task("genpybind",self.to_nodes(self.source),out)
	task.features=self.features
	task.module=module
	task.genpybind_tags=self.to_list(getattr(self,"genpybind_tags",[]))
	task.includes=self.to_list(getattr(self,"includes",[]))
	task.genpybind=self.env.GENPYBIND
	self.source=[out]
class genpybind(Task.Task):
	quiet=True
	color="PINK"
	scan=scan_impl
	@staticmethod
	def keyword():
		return"Analyzing"
	def run(self):
		if not self.inputs:
			return
		args=self.find_genpybind()+self._arguments(resource_dir=self.env.GENPYBIND_RESOURCE_DIR)
		output=self.run_genpybind(args)
		pasteable_command=join_args(args)
		(output_node,)=self.outputs
		output_node.write("// {}\n{}\n".format(pasteable_command.replace("\n","\n// "),output))
	def find_genpybind(self):
		return self.genpybind
	def run_genpybind(self,args):
		bld=self.generator.bld
		kwargs=dict(cwd=bld.variant_dir)
		if hasattr(bld,"log_command"):
			bld.log_command(args,kwargs)
		else:
			Logs.debug("runner: {!r}".format(args))
		proc=subprocess.Popen(args,stdout=subprocess.PIPE,stderr=subprocess.PIPE,**kwargs)
		stdout,stderr=proc.communicate()
		if not isinstance(stdout,str):
			stdout=stdout.decode(sys.stdout.encoding,errors="replace")
		if not isinstance(stderr,str):
			stderr=stderr.decode(sys.stderr.encoding,errors="replace")
		if proc.returncode!=0:
			bld.fatal("genpybind returned {code} during the following call:""\n{command}\n\n{stdout}\n\n{stderr}".format(code=proc.returncode,command=join_args(args),stdout=stdout,stderr=stderr,))
		if stderr.strip():
			Logs.debug("non-fatal warnings during genpybind run:\n{}".format(stderr))
		return stdout
	def _include_paths(self):
		return self.generator.to_incnodes(self.includes+self.env.INCLUDES)
	def _inputs_as_relative_includes(self):
		include_paths=self._include_paths()
		relative_includes=[]
		for node in self.inputs:
			for inc in include_paths:
				if node.is_child_of(inc):
					relative_includes.append(node.path_from(inc))
					break
			else:
				self.generator.bld.fatal("could not resolve {}".format(node))
		return relative_includes
	def _arguments(self,genpybind_parse=None,resource_dir=None):
		args=[]
		relative_includes=self._inputs_as_relative_includes()
		is_cxx="cxx"in self.features
		args.extend(["--genpybind-module",self.module])
		if self.genpybind_tags:
			args.extend(["--genpybind-tag"]+self.genpybind_tags)
		if relative_includes:
			args.extend(["--genpybind-include"]+relative_includes)
		if genpybind_parse:
			args.extend(["--genpybind-parse",genpybind_parse])
		args.append("--")
		args.extend(node.abspath()for node in self.inputs)
		args.append("--")
		args.append("-D__GENPYBIND__")
		args.append("-xc++"if is_cxx else"-xc")
		has_std_argument=False
		for flag in self.env["CXXFLAGS"if is_cxx else"CFLAGS"]:
			flag=flag.replace("-std=gnu","-std=c")
			if flag.startswith("-std=c"):
				has_std_argument=True
			args.append(flag)
		if not has_std_argument:
			args.append("-std=c++14")
		args.extend("-I{}".format(n.abspath())for n in self._include_paths())
		args.extend("-D{}".format(p)for p in self.env.DEFINES)
		if resource_dir:
			args.append("-resource-dir={}".format(resource_dir))
		return args
