# encoding: utf-8
# WARNING! Do not edit! https://waf.io/book/index.html#_obtaining_the_waf_file

import re,os
from waflib.Task import Task
from waflib.TaskGen import extension
from waflib import Errors,Context,Logs
class protoc(Task):
	run_str='${PROTOC} ${PROTOC_FL:PROTOC_FLAGS} ${PROTOC_ST:INCPATHS} ${PROTOC_ST:PROTOC_INCPATHS} ${PROTOC_ST:PROTOC_EXTINCPATHS} ${SRC[0].bldpath()}'
	color='BLUE'
	ext_out=['.h','pb.cc','.py','.java']
	def scan(self):
		node=self.inputs[0]
		nodes=[]
		names=[]
		seen=[]
		search_nodes=[]
		if not node:
			return(nodes,names)
		if'cxx'in self.generator.features:
			search_nodes=self.generator.includes_nodes
		if'py'in self.generator.features or'javac'in self.generator.features:
			for incpath in getattr(self.generator,'protoc_includes',[]):
				incpath_node=self.generator.path.find_node(incpath)
				if incpath_node:
					search_nodes.append(incpath_node)
				else:
					incpath_node=self.generator.bld.path.find_node(incpath)
					if incpath_node:
						search_nodes.append(incpath_node)
					else:
						raise Errors.WafError('protoc: include path %r does not exist'%incpath)
		def parse_node(node):
			if node in seen:
				return
			seen.append(node)
			code=node.read().splitlines()
			for line in code:
				m=re.search(r'^import\s+"(.*)";.*(//)?.*',line)
				if m:
					dep=m.groups()[0]
					for incnode in search_nodes:
						found=incnode.find_resource(dep)
						if found:
							nodes.append(found)
							parse_node(found)
						else:
							names.append(dep)
		parse_node(node)
		for deppath in nodes:
			self.env.append_unique('INCPATHS',deppath.parent.bldpath())
		return(nodes,names)
@extension('.proto')
def process_protoc(self,node):
	incdirs=[]
	out_nodes=[]
	protoc_flags=[]
	self.env.PROTOC_FLAGS=self.to_list(self.env.PROTOC_FLAGS)
	if'cxx'in self.features:
		cpp_node=node.change_ext('.pb.cc')
		hpp_node=node.change_ext('.pb.h')
		self.source.append(cpp_node)
		out_nodes.append(cpp_node)
		out_nodes.append(hpp_node)
		protoc_flags.append('--cpp_out=%s'%node.parent.get_bld().bldpath())
	if'py'in self.features:
		py_node=node.change_ext('_pb2.py')
		self.source.append(py_node)
		out_nodes.append(py_node)
		protoc_flags.append('--python_out=%s'%node.parent.get_bld().bldpath())
	if'javac'in self.features:
		if not node.parent.get_bld()in self.javac_task.srcdir:
			self.javac_task.srcdir.append(node.parent.get_bld())
		protoc_flags.append('--java_out=%s'%node.parent.get_bld().bldpath())
		node.parent.get_bld().mkdir()
	tsk=self.create_task('protoc',node,out_nodes)
	tsk.env.append_value('PROTOC_FLAGS',protoc_flags)
	if'javac'in self.features:
		self.javac_task.set_run_after(tsk)
	for incpath in getattr(self,'protoc_includes',[]):
		incpath_node=self.path.find_node(incpath)
		if incpath_node:
			incdirs.append(incpath_node.bldpath())
		else:
			incpath_node=self.bld.path.find_node(incpath)
			if incpath_node:
				incdirs.append(incpath_node.bldpath())
			else:
				raise Errors.WafError('protoc: include path %r does not exist'%incpath)
	tsk.env.PROTOC_INCPATHS=incdirs
	tsk.env.PROTOC_EXTINCPATHS=getattr(self,'protoc_extincludes',[])
	tsk.env.prepend_value('INCPATHS',node.parent.bldpath())
	use=getattr(self,'use','')
	if not'PROTOBUF'in use:
		self.use=self.to_list(use)+['PROTOBUF']
def configure(conf):
	conf.check_cfg(package='protobuf',uselib_store='PROTOBUF',args=['--cflags','--libs'])
	conf.find_program('protoc',var='PROTOC')
	conf.start_msg('Checking for protoc version')
	protocver=conf.cmd_and_log(conf.env.PROTOC+['--version'],output=Context.BOTH)
	protocver=''.join(protocver).strip()[protocver[0].rfind(' ')+1:]
	conf.end_msg(protocver)
	conf.env.PROTOC_MAJOR=protocver[:protocver.find('.')]
	conf.env.PROTOC_ST='-I%s'
	conf.env.PROTOC_FL='%s'
